@@ -76,7 +76,7 @@ class DeltaCenterArguments:
         default=None,
         metadata={"help": "the ratio of delta parameters in the model"}
     )
-    usage_code: Optional[str] = field(
+    usage: Optional[str] = field(
         default="",
         metadata={"help": "the usage code of the model"}
     )
@@ -90,25 +90,26 @@ class DeltaCenterArguments:
 class SaveLoadMixin:
     def add_configs_when_saving(self,):
         self.config.backbone_class = self.backbone_model.__class__.__name__
-        self.config.backbone_checkpoint_name = os.path.split(self.backbone_model.config._name_or_path.strip("/"))[-1]
+        if hasattr(self.backbone_model, "config"):
+            self.config.backbone_checkpoint_name = os.path.split(self.backbone_model.config._name_or_path.strip("/"))[-1]
         self.config.backbone_hash = gen_model_hash(self.backbone_model)
 
 
 
-
     def save_finetuned(
         self,
         finetuned_delta_path: Optional[Union[str, os.PathLike]] = "./delta_checkpoints/",
         save_config: bool = True,
         state_dict: Optional[dict] = None,
         save_function: Callable = torch.save,
-        push_to_dc: bool = True,
+        push_to_dc: bool = False,
         center_args: Optional[Union[DeltaCenterArguments, dict]] = dict(),
         center_args_pool: Optional[dict] = dict(),
         list_tags: Optional[List] = list(),
         dict_tags: Optional[Dict] = dict(),
         delay_push: bool = False,
-        usage_code: Optional[str] = "",
+        test_result = None,
+        usage: Optional[str] = "",
     ):
         r"""
         Save a model and its configuration file to a directory, so that it can be re-loaded using the
@@ -172,29 +173,53 @@ class SaveLoadMixin:
          # Save the config
         if save_config:
             self.config.save_finetuned(save_directory)
+        
 
 
-        logger.info("\n"+"*"*30+f"\nYou delta models has been saved locally to:\n\t{os.path.abspath(save_directory)}"
+        
+
+
+        logger.info("\n"+"*"*30+f"\nYou delta models has been saved locally to:\t{os.path.abspath(save_directory)}"
                  )
+        self.compute_saving(output_model_file)
 
+        state_dict_total_params = sum(p.numel() for p in state_dict.values())
+        other_tags={}
+        other_tags.update({'state_dict_total_params(M)':state_dict_total_params/1024/1024})
+        other_tags.update({'test_result':test_result})
         if push_to_dc:
             logger.info("Creating yaml file for delta center")
-            self.create_yml(save_directory, final_center_args, list_tags, dict_tags)
+            self.create_yml(save_directory, final_center_args, list_tags, dict_tags, other_tags)
 
             if not delay_push:
                 OssClient.upload(base_dir=save_directory)
             else:
                 logger.info(f"Delay push: you can push it to the delta center later using \n\tpython -m DeltaCenter upload {os.path.abspath(save_directory)}\n"
                     +"*"*30)
+        else:
+            logger.info("We encourage users to push their final and public models to delta center to share them with the community!")
+
+    def compute_saving(self, output_model_file):
+        import os
+        stats = os.stat(output_model_file)
+        if stats.st_size > (1024**3):
+            unit = 'GB'
+            value = stats.st_size/(1024**3)
+        else:
+            unit = 'MB'
+            value = stats.st_size/(1024**2)
+        logger.info("The state dict size is {:.3f} {}".format(value, unit))
 
 
 
 
-    def create_yml(self, save_dir, config, list_tags=list(), dict_tags=dict()):
+    def create_yml(self, save_dir, config, list_tags=list(), dict_tags=dict(),other_tags=None):
         f = open("{}/config.yml".format(save_dir), 'w')
         config_dict = vars(config)
         config_dict['dict_tags'] = dict_tags
         config_dict['list_tags'] = list_tags
+        if other_tags is not None:
+            config_dict.update(other_tags)
         yaml.safe_dump(config_dict, f)
         f.close()
 
@@ -376,7 +401,7 @@ class SaveLoadMixin:
 
     def create_default_usage(self, name):
         usage_str = """from opendelta import AutoDeltaModel\n""" + \
-            """delta_model = AutoDeltaModel.from_finetuned({name_with_userid}, backbone_model=model)\n""" + \
+            """delta_model = AutoDeltaModel.from_finetuned('{name_with_userid}', backbone_model=model)\n""" + \
             """delta_model.freeze_module() # if you are going to further train it \n""" + \
             """delta_model.log()"""
         return usage_str
@@ -386,7 +411,7 @@ class SaveLoadMixin:
         """
 
         reponame = ""
-        reponame += kwargs["model_path_public"].split("/")[-1]+"_" if kwargs['model_path_public'] is not None else kwargs['backbone_model']
+        reponame += kwargs["backbone_model_path_public"].split("/")[-1]+"_" if kwargs['backbone_model_path_public'] is not None else kwargs['backbone_model']
         reponame += kwargs["delta_type"]+"_" if kwargs["delta_type"] is not None else ""
 
         # tasks
